@@ -1,3 +1,5 @@
+from functools import lru_cache
+
 from django import template
 from django.conf import settings
 from django.forms import boundfield
@@ -6,7 +8,6 @@ from django.template import Context
 from django.template.loader import get_template
 from django.utils.safestring import mark_safe
 
-from crispy_forms.compatibility import lru_cache
 from crispy_forms.exceptions import CrispyError
 from crispy_forms.utils import TEMPLATE_PACK, flatatt
 
@@ -124,3 +125,34 @@ def as_crispy_field(field, template_pack=TEMPLATE_PACK, label_class="", field_cl
 @register.filter(name="flatatt")
 def flatatt_filter(attrs):
     return mark_safe(flatatt(attrs))
+
+
+@register.filter
+def optgroups(field):
+    """
+    A template filter to help rendering of fields with optgroups.
+
+    Returns:
+        A tuple of label, option, index
+
+        label: Group label for grouped optgroups (`None` if inputs are not
+               grouped).
+        option: A dict containing information to render the option::
+            {
+                "name": "checkbox_select_multiple",
+                "value": 1,
+                "label": 1,
+                "selected": False,
+                "index": "0",
+                "attrs": {"id": "id_checkbox_select_multiple_0"},
+                "type": "checkbox",
+                "template_name": "django/forms/widgets/checkbox_option.html",
+                "wrap_label": True,
+            }
+        index: Group index`
+    """
+    id_ = field.field.widget.attrs.get("id") or field.auto_id
+    attrs = {"id": id_} if id_ else {}
+    attrs = field.build_widget_attrs(attrs)
+    values = field.field.widget.format_value(field.value())
+    return field.field.widget.optgroups(field.html_name, values, attrs)
