@@ -11,9 +11,11 @@ from crispy_forms.bootstrap import (
     AccordionGroup,
     Alert,
     AppendedText,
+    Container,
     FieldWithButtons,
     InlineCheckboxes,
     InlineRadios,
+    Modal,
     PrependedAppendedText,
     PrependedText,
     StrictButton,
@@ -22,10 +24,10 @@ from crispy_forms.bootstrap import (
 )
 from crispy_forms.helper import FormHelper
 from crispy_forms.layout import HTML, Field, Layout, MultiWidgetField
-from crispy_forms.tests.utils import contains_partial
+from crispy_forms.tests.utils import contains_partial, parse_expected, parse_form
 from crispy_forms.utils import render_crispy_form
 
-from .conftest import only_bootstrap, only_bootstrap4
+from .conftest import only_bootstrap, only_bootstrap3, only_bootstrap4
 from .forms import (
     CheckboxesSampleForm,
     CustomCheckboxSelectMultiple,
@@ -34,7 +36,6 @@ from .forms import (
     SampleForm,
     SampleFormCustomWidgets,
 )
-from .utils import parse_expected, parse_form
 
 
 def test_field_with_custom_template():
@@ -224,10 +225,21 @@ class TestBootstrapLayoutObjects:
                 PrependedAppendedText("email", "@", "gmail.com", css_class="form-control-lg")
             )
             html = render_crispy_form(test_form)
-
             assert 'class="form-control-lg' in html
             assert contains_partial(html, '<span class="input-group-text"/>')
 
+    @only_bootstrap4
+    def test_prepended_wrapper_class(self, settings):
+        test_form = SampleForm()
+        test_form.helper = FormHelper()
+        test_form.helper.layout = Layout(
+            PrependedAppendedText("email", "@", "gmail.com", wrapper_class="wrapper class"),
+            PrependedAppendedText("email", "@", "gmail.com"),
+        )
+        html = render_crispy_form(test_form)
+        assert html.count('<div id="div_id_email" class="form-group">') == 1
+        assert html.count('<div id="div_id_email" class="form-group wrapper class">') == 1
+
     @only_bootstrap4
     def test_prepended_appended_text_in_select(self, settings):
         test_form = SampleForm()
@@ -453,32 +465,12 @@ class TestBootstrapLayoutObjects:
                 StrictButton("Test", type="submit", name="whatever", value="something"),
                 css_class="extra",
                 autocomplete="off",
+                input_size="input-group-sm",
             )
         )
-        html = render_crispy_form(form)
-
-        form_group_class = "control-group"
-        if settings.CRISPY_TEMPLATE_PACK in ("bootstrap3", "bootstrap4"):
-            form_group_class = "form-group"
-
-        assert html.count('class="%s extra"' % form_group_class) == 1
-        assert html.count('autocomplete="off"') == 1
-        assert html.count('class="span4') == 1
-        assert html.count('id="go-button"') == 1
-        assert html.count("Go!") == 1
-        assert html.count("No!") == 1
-        assert html.count('class="btn"') == 2
-        assert html.count('class="btn extra"') == 1
-        assert html.count('type="submit"') == 1
-        assert html.count('name="whatever"') == 1
-        assert html.count('value="something"') == 1
-
-        if settings.CRISPY_TEMPLATE_PACK == "bootstrap":
-            assert html.count('class="input-append"') == 1
-        elif settings.CRISPY_TEMPLATE_PACK == "bootstrap3":
-            assert html.count('class="input-group-btn') == 1
-        elif settings.CRISPY_TEMPLATE_PACK == "bootstrap4":
-            assert html.count('class="input-group-append') == 1
+        assert parse_form(form) == parse_expected(
+            f"{settings.CRISPY_TEMPLATE_PACK}/test_layout_objects/test_field_with_buttons.html"
+        )
 
     def test_hidden_fields(self):
         form = SampleForm()
@@ -555,3 +547,62 @@ class TestBootstrapLayoutObjects:
         )
         form.helper.layout = Layout("radio")
         assert parse_form(form) == parse_expected("bootstrap4/test_layout_objects/test_grouped_radios_failing.html")
+
+    def test_non_ascii_chars_in_container_name(self):
+        """
+        Test if non-ASCII characters are saved as css_id property.
+        """
+        name = "テスト"
+        test_container = Container(name, "val1", "val2")
+        assert test_container.css_id == name
+
+    @only_bootstrap3
+    def test_modal_no_kwargs(self):
+        form = SampleForm()
+        form.helper = FormHelper()
+        form.helper.layout = Layout(Modal(Field("first_name")))
+
+        assert parse_form(form) == parse_expected("bootstrap3/test_layout_objects/bootstrap_modal_no_kwargs.html")
+
+    @only_bootstrap3
+    def test_modal_with_kwargs(self):
+        form = SampleForm()
+        form.helper = FormHelper()
+        form.helper.layout = Layout(
+            Modal(
+                Field("first_name"),
+                css_id="id_test",
+                css_class="test-class",
+                title="This is my modal",
+                title_id="id_title_test",
+                title_class="text-center",
+            )
+        )
+
+        assert parse_form(form) == parse_expected("bootstrap3/test_layout_objects/bootstrap_modal_with_kwargs.html")
+
+    @only_bootstrap4
+    def test_bs4_modal_no_kwargs(self):
+        form = SampleForm()
+        form.helper = FormHelper()
+        form.helper.layout = Layout(Modal(Field("first_name")))
+
+        print(parse_form(form))
+        assert parse_form(form) == parse_expected("bootstrap4/test_layout_objects/bootstrap_modal_no_kwargs.html")
+
+    @only_bootstrap4
+    def test_bs4_modal_with_kwargs(self):
+        form = SampleForm()
+        form.helper = FormHelper()
+        form.helper.layout = Layout(
+            Modal(
+                Field("first_name"),
+                css_id="id_test",
+                css_class="test-class",
+                title="This is my modal",
+                title_id="id_title_test",
+                title_class="text-center",
+            )
+        )
+
+        assert parse_form(form) == parse_expected("bootstrap4/test_layout_objects/bootstrap_modal_with_kwargs.html")
