@@ -1,5 +1,12 @@
+import os
+from pathlib import Path
+
 from django.test.html import Element, parse_html
 
+from crispy_forms.utils import render_crispy_form
+
+TEST_DIR = os.path.dirname(os.path.abspath(__file__))
+
 
 def contains_partial(haystack, needle, ignore_needle_children=False):
     """Search for a html element with at least the corresponding elements
@@ -20,3 +27,14 @@ def contains_partial(haystack, needle, ignore_needle_children=False):
         for child in haystack.children
         if isinstance(child, Element)
     )
+
+
+def parse_expected(expected_file):
+    test_file = Path(TEST_DIR) / "results" / expected_file
+    with test_file.open() as f:
+        return parse_html(f.read())
+
+
+def parse_form(form):
+    html = render_crispy_form(form)
+    return parse_html(html)
